# -*- coding:utf-8 -*-

from keras.layers import Activation, Conv2D, Dense, Flatten, MaxPooling2D
from keras.models import Sequential
from keras.preprocessing.image import ImageDataGenerator

import pathlib


def create_model(class_count):
    model = Sequential()
    model.add(Conv2D(64, (3, 3), activation="relu", input_shape=(32, 32, 3)))
    model.add(MaxPooling2D(pool_size=(2, 2)))
    model.add(Conv2D(128, (3, 3), activation="relu"))
    model.add(MaxPooling2D(pool_size=(2, 2)))
    model.add(Flatten())
    model.add(Dense(128, activation="relu"))
    model.add(Dense(class_count, activation="softmax"))
    return model


def main(opts):
    # create model
    data_path = pathlib.Path(opts.data)
    cls = [d for d in data_path.iterdir() if d.is_dir()]
    print(cls)
    model = create_model(len(cls))
    model.compile(optimizer='adam',
            loss='categorical_crossentropy',
            metrics=['accuracy'])

    # dataset
    image_gen = ImageDataGenerator(
            rescale=1./255,
            rotation_range=20,
            width_shift_range=0.2,
            height_shift_range=0.2,
            horizontal_flip=True,
            vertical_flip=True,
            validation_split=0.3)
    train_generator = image_gen.flow_from_directory(
            opts.data, target_size=(32, 32), batch_size=16, subset='training')
    test_generator = image_gen.flow_from_directory(
            opts.data, target_size=(32, 32), batch_size=16, subset='validation')

    # train&test
    model.fit_generator(train_generator,
            steps_per_epoch=100,
            epochs=5,
            validation_data=test_generator,
            validation_steps=1)

    # save model
    model.save(opts.result)


def parse_args():
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument('--data', required=True, help='path of image dataset')
    parser.add_argument('--result', required=True, help='path for result model data')
    return parser.parse_args()


if __name__ == "__main__":
    main(parse_args())
